@@ -1,18 +1,21 @@
 import contextlib
 from pathlib import Path
-from typing import Optional, Dict, List, Iterable, Union, Tuple
+from typing import Optional, Dict, Union, Tuple
 
-from nonebot.adapters import Bot, Event
+from nonebot.adapters import Event
 from nonebot.log import logger
 from nonebot.matcher import current_event
 from nonebot.permission import Permission
 from nonebot.plugin.export import export
 
 from .check import check, get_permission_group_by_event
-from .core import get, get_namespace, PermissionGroup
+from .core import get, get_namespace, PermissionGroup, decorate_permission, parse_qualified_group_name
 
 plugins: Dict[str, "PluginHandler"] = {}
 
+_sentinel = object()
+Designator = Union[Event, str, None]
+
 
 @export()
 def register(plugin_name: str) -> "PluginHandler":
@@ -33,21 +36,22 @@ def register(plugin_name: str) -> "PluginHandler":
 
 
 class PluginHandler:
-    _sentinel = object()
-
     def __init__(self, name: str):
         self.name = name
         self.preset_: Optional[Path] = None
+        self.decorate_ = False
         self.check_root_ = False
 
-    def preset(self, preset: Path):
+    def preset(self, preset: Path, decorate: bool = False):
         """
         设置预设权限组，会被加载到插件名对应的名称空间。
 
         :param preset: 包含权限组的文件路径。
+        :param decorate: 是否自动修饰预设包含的权限名。
         :return: self
         """
         self.preset_ = preset
+        self.decorate_ = decorate
         return self
 
     def check_root(self):
@@ -74,8 +78,8 @@ class PluginHandler:
         :param check_root: 如果传入布尔值，则替代之前 self.check_root() 的设定。
         :return: 权限检查器，可以直接传递给 nonebot 事件响应器。
         """
-        full = self._parse_perm(perm)
-        if check_root is self._sentinel:
+        full = decorate_permission(self.name, perm)
+        if check_root is _sentinel:
             check_root = self.check_root_
         if check_root:
             full.insert(0, self.name)
@@ -99,26 +103,12 @@ class PluginHandler:
         :param event: 事件，默认为当前正在处理的事件。
         :return: 检查结果。
         """
-        deprecate = False
-        if perm and isinstance(perm[0], Bot):
-            perm = perm[1:]
-            deprecate = True
-        if perm and isinstance(perm[0], Event):
-            if event is not None:
-                raise TypeError("has() got multiple values for argument 'event'")
-            event, *perm = perm
-            deprecate = True
-        if deprecate:
-            import warnings
-            warnings.warn('Positional parameters "bot" and "event" are deprecated and will be removed soon.',
-                          DeprecationWarning, stacklevel=2)
-
         if event is None:
             event = current_event.get()
-        full = self._parse_perm(perm)
+        full = decorate_permission(self.name, perm)
         return all(check(event, px) for px in full)
 
-    def add_permission(self, designator: Union[Event, str, None], perm: str, *,
+    def add_permission(self, designator: Designator, perm: str = _sentinel, *,
                        comment: str = None, create_group: bool = True) -> bool:
         """
         向权限组添加一项权限。会修饰权限名。
@@ -133,8 +123,11 @@ class PluginHandler:
         :raise KeyError: 权限组不存在，并且指定为不自动创建。
         :raise TypeError: 权限组不可修改。
         """
+        if perm is _sentinel:
+            designator, perm = None, designator
+
         group_ = self._get_or_create_group(designator, create_group, True)
-        perm = self._parse_perm([perm])[0]
+        [perm] = decorate_permission(self.name, [perm])
         with contextlib.suppress(ValueError):
             group_.remove('-' + perm)
         try:
@@ -143,7 +136,7 @@ class PluginHandler:
         except ValueError:
             return False
 
-    def remove_permission(self, designator: Union[Event, str, None], perm: str, *,
+    def remove_permission(self, designator: Designator, perm: str = _sentinel, *,
                           comment: str = None, create_group: bool = True) -> bool:
         """
         从权限组去除一项权限。会修饰权限名。
@@ -158,8 +151,11 @@ class PluginHandler:
         :raise KeyError: 权限组不存在，并且指定为不自动创建。
         :raise TypeError: 权限组不可修改。
         """
+        if perm is _sentinel:
+            designator, perm = None, designator
+
         group_ = self._get_or_create_group(designator, create_group, True)
-        perm = self._parse_perm([perm])[0]
+        [perm] = decorate_permission(self.name, [perm])
         with contextlib.suppress(ValueError):
             group_.remove(perm)
         try:
@@ -168,7 +164,8 @@ class PluginHandler:
         except ValueError:
             return False
 
-    def reset_permission(self, designator: Union[Event, str, None], perm: str, *, allow_missing: bool = True) -> bool:
+    def reset_permission(self, designator: Designator, perm: str = _sentinel, *,
+                         allow_missing: bool = True) -> bool:
         """
         把权限组中关于一项权限的描述恢复默认。会修饰权限名。
 
@@ -181,10 +178,13 @@ class PluginHandler:
         :raise KeyError: 权限组不存在，并且指定为不静默忽略。
         :raise TypeError: 权限组不可修改。
         """
+        if perm is _sentinel:
+            designator, perm = None, designator
+
         group_ = self._get_or_create_group(designator, allow_missing, False)
         if group_ is None:
             return False
-        perm = self._parse_perm([perm])[0]
+        [perm] = decorate_permission(self.name, [perm])
         # noinspection PyUnusedLocal
         modified = False
         with contextlib.suppress(ValueError):
@@ -196,7 +196,7 @@ class PluginHandler:
             modified = True
         return modified
 
-    def add_item(self, designator: Union[Event, str, None], item: str, *,
+    def add_item(self, designator: Designator, item: str = _sentinel, *,
                  comment: str = None, create_group: bool = True) -> bool:
         """
         向权限组添加权限描述。会修饰权限名。
@@ -209,18 +209,23 @@ class PluginHandler:
         :raise KeyError: 权限组不存在，并且指定为不自动创建。
         :raise TypeError: 权限组不可修改。
         """
+        if item is _sentinel:
+            designator, item = None, designator
+
         group_ = self._get_or_create_group(designator, create_group, True)
         if item.startswith('-'):
-            item = '-' + self._parse_perm([item[1:]])[0]
+            [item] = decorate_permission(self.name, [item[1:]])
+            item = '-' + item
         else:
-            item = self._parse_perm([item])[0]
+            [item] = decorate_permission(self.name, [item])
         try:
             group_.add(item, comment)
             return True
         except ValueError:
             return False
 
-    def remove_item(self, designator: Union[Event, str, None], item: str, *, allow_missing: bool = True) -> bool:
+    def remove_item(self, designator: Designator, item: str = _sentinel, *,
+                    allow_missing: bool = True) -> bool:
         """
         从权限组中移除权限描述。会修饰权限名。
 
@@ -231,21 +236,74 @@ class PluginHandler:
         :raise KeyError: 权限组不存在，并且指定为不静默忽略。
         :raise TypeError: 权限组不可修改。
         """
+        if item is _sentinel:
+            designator, item = None, designator
+
         group_ = self._get_or_create_group(designator, allow_missing, False)
         if group_ is None:
             return False
         if item.startswith('-'):
-            item = '-' + self._parse_perm([item[1:]])[0]
+            [item] = decorate_permission(self.name, [item[1:]])
+            item = '-' + item
         else:
-            item = self._parse_perm([item])[0]
+            [item] = decorate_permission(self.name, [item])
         try:
             group_.remove(item)
             return True
         except ValueError:
             return False
 
+    def add_inheritance(self, designator: Designator, target: Designator = _sentinel, *,
+                        comment: str = None, create_group: bool = True) -> bool:
+        """
+        向权限组添加继承关系。
+
+        :param designator: 待修改权限组的指示符。
+        :param target: 需继承权限组的指示符。<em>如省略名称空间则默认为当前插件。</em>
+        :param comment: 注释。
+        :param create_group: 如果待修改权限组不存在，是否自动创建。
+        :return: 是否确实添加了，如果权限组中已有指定继承关系则返回 False 。
+        :raise KeyError: 待修改权限组不存在，并且指定为不自动创建；或需继承的权限组不存在。
+        :raise TypeError: 权限组不可修改。
+        """
+        if target is _sentinel:
+            designator, target = None, designator
+
+        ancestor = self._get_or_create_group(target, False, False, self.name)
+        group_ = self._get_or_create_group(designator, create_group, True)
+        try:
+            group_.add_inheritance(ancestor, comment)
+            return True
+        except ValueError:
+            return False
+
+    def remove_inheritance(self, designator: Designator, target: Designator = _sentinel, *,
+                           allow_missing: bool = True) -> bool:
+        """
+        从权限组中移除继承关系。
+
+        :param designator: 待修改权限组的指示符。
+        :param target: 需移除继承权限组的指示符。<em>如省略名称空间则默认为当前插件。</em>
+        :param allow_missing: 如果待修改权限组不存在，是否静默忽略。
+        :return: 是否确实移除了，如果权限组中没有指定继承关系则返回 False 。
+        :raise KeyError: 待修改权限组不存在，并且指定为不静默忽略；或需移除继承的权限组不存在。
+        :raise TypeError: 权限组不可修改。
+        """
+        if target is _sentinel:
+            designator, target = None, designator
+
+        group_ = self._get_or_create_group(designator, allow_missing, False)
+        if group_ is None:
+            return False
+        ancestor = self._get_or_create_group(target, False, False, self.name)
+        try:
+            group_.remove_inheritance(ancestor)
+            return True
+        except ValueError:
+            return False
+
     @classmethod
-    def add_group(cls, designator: Union[Event, str, None], *, comment: str = None):
+    def add_group(cls, designator: Designator = None, *, comment: str = None):
         """
         创建权限组。
 
@@ -258,7 +316,7 @@ class PluginHandler:
         get_namespace(namespace, False).add_group(group, comment)
 
     @classmethod
-    def remove_group(cls, designator: Union[Event, str, None], *, force: bool = False):
+    def remove_group(cls, designator: Designator = None, *, force: bool = False):
         """
         移除权限组。
 
@@ -272,7 +330,8 @@ class PluginHandler:
         get_namespace(namespace, False).remove_group(group, force)
 
     @classmethod
-    def _parse_designator(cls, designator: Union[Event, str, None]) -> Tuple[str, Union[str, int]]:
+    def _parse_designator(cls, designator: Designator, default_namespace: str = 'global'
+                          ) -> Tuple[str, Union[str, int]]:
         if designator is None:
             designator = current_event.get()
         if isinstance(designator, Event):
@@ -281,20 +340,13 @@ class PluginHandler:
                 return result
             raise ValueError('Unrecognized event type: ' + designator.get_event_name())
         if isinstance(designator, str):
-            designator_split = designator.split(':', maxsplit=1)
-            if len(designator_split) == 1:
-                return 'global', designator_split[0]
-            namespace, group = designator_split
-            if namespace in ['group', 'user']:
-                with contextlib.suppress(ValueError):
-                    group = int(group)
-            return namespace, group
+            return parse_qualified_group_name(designator, default_namespace)
         raise ValueError(f'Invalid designator: {type(designator)}')
 
     @classmethod
-    def _get_or_create_group(cls, designator: Union[Event, str, None], silent: bool, create: bool
+    def _get_or_create_group(cls, designator: Designator, silent: bool, create: bool, default_namespace: str = 'global'
                              ) -> Optional[PermissionGroup]:
-        namespace, group = cls._parse_designator(designator)
+        namespace, group = cls._parse_designator(designator, default_namespace)
         group_, found = get(namespace, group)
         if not found:
             if not silent:
@@ -304,17 +356,3 @@ class PluginHandler:
             get_namespace(namespace, False).add_group(group)
             group_, _ = get(namespace, group)
         return group_
-
-    def _parse_perm(self, perm: Iterable[str]) -> List[str]:
-        result = []
-        for p in perm:
-            if not p:
-                result.append(self.name)
-            elif p.startswith('/'):
-                result.append(p[1:])
-            elif p.startswith('.'):
-                prev = result[-1] if result else self.name
-                result.append(prev + p)
-            else:
-                result.append(self.name + '.' + p)
-        return result
